DEFAULT_API_HOST = "https://api.vanta.ge"
API_HOST_VERSION = "v1"

DEFAULT_AUTH_HOST = "https://auth.vanta.ge"
AUTH_ENDPOINT = "/oauth/token"

DEFAULT_ENCODING = "utf-8"

METADATA_PREFIX = "meta_"
METADATA_ORDERED_PREFIX = "meta_ordered_"
